@@ -178,6 +178,15 @@ static int get_dev_num(const char *path, dev_t *dev_num)
 	return 0;
 }
 
+int get_dev_by_name(const char *device, dev_t *dev)
+{
+	char nbuf[4096];
+
+	snprintf(nbuf, sizeof(nbuf), "/sys/block/%s/dev", basename(device));
+
+	return get_dev_num(nbuf, dev);
+}
+
 int get_dev_start(const char *path, __u32 *start)
 {
 	char nbuf[4096];
@@ -258,21 +267,6 @@ close_dir:
 	return -1;
 }
 
-static int is_cookie_supported(void)
-{
-	struct utsname buf;
-	int major, minor;
-
-	uname(&buf);
-	// >= 2.6.32-042stab061.1
-	if (sscanf(buf.release, "%*d.%*d.%*d-%dstab%d.%*d", &major, &minor) != 2) {
-		ploop_err(0, "Can't parse kernel version %s", buf.release);
-		return 0;
-	}
-
-	return (major > 42 || (major == 42 && minor >= 61));
-}
-
 /* Find device by base delta and return the name
  * return: -1 on error
  *	    0 found
@@ -316,7 +310,7 @@ int ploop_find_dev(const char *component_name, const char *delta,
 				de->d_name);
 		err = read_line_quiet(fname, image, sizeof(image));
 		if (err) {
-			if (err == ENOENT)
+			if (err == ENOENT || err == ENODEV)
 				continue;
 
 			ploop_err(err, "Can't open or read %s", fname);
@@ -330,7 +324,7 @@ int ploop_find_dev(const char *component_name, const char *delta,
 				de->d_name);
 		err = read_line_quiet(fname, cookie, sizeof(cookie));
 		if (err) {
-			if ((err == ENOENT) && is_cookie_supported())
+			if (err == ENOENT || err == ENODEV)
 				/* This is not an error, but a race between
 				 * mount and umount: device is being removed
 				 */
