@@ -27,7 +27,10 @@ while test -n "$1"; do
 	shift
 done
 
-RPM_SPEC=ploop.spec
+test "$build" = "yes" && clean="yes"
+
+NAME=ploop
+RPM_SPEC=${NAME}.spec
 
 # Try to figure out version from git
 GIT_DESC=$(git describe --tags | sed s'/^[^0-9]*-\([0-9].*\)$/\1/')
@@ -44,6 +47,12 @@ read_spec() {
 }
 read_spec
 
+# Store original spec
+if test "$clean" = "yes"; then
+	cp -a $RPM_SPEC .$RPM_SPEC.$$
+	trap "mv -f .$RPM_SPEC.$$ $RPM_SPEC" EXIT
+fi
+
 # Set version/release in spec from git
 if test "$GIT_VR" != "$SPEC_VR"; then
 	test -z "$verbose" || echo "Changing $RPM_SPEC:"
@@ -62,12 +71,17 @@ fi
 test -z "$verbose" || \
 	grep -E -H '^Version:|^%define rel|^Source:|^%setup' $RPM_SPEC
 
-# Set version in configure.ac from spec
-read_spec
-SPEC_VR=$(echo $SPEC_VR | sed 's/-1$//')
-
 test "$build" = "yes" || exit 0
 make rpms || exit 1
 
+# Remove dist tarball
+test "$clean" = "yes" && rm -f ${NAME}-${GIT_VR}.tar.bz2
+
 test -z "$install" && exit 0
-sudo rpm -${install}hv $(rpm --eval %{_rpmdir}/%{_arch})/ploop-*${GIT_VR}*.rpm
+sudo rpm -${install}hv $(rpm --eval %{_rpmdir}/%{_arch})/${NAME}-*${GIT_VR}*.rpm
+
+# Remove rpms
+if test "$clean" = "yes"; then
+	rm -f $(rpm --eval %{_rpmdir}/%{_arch})/${NAME}-*${GIT_VR}*.rpm
+	rm -f $(rpm --eval %{_srcrpmdir})/${NAME}-*${GIT_VR}*.src.rpm
+fi
