@@ -57,22 +57,26 @@ static void usage_summary(void)
 
 static void usage_init(void)
 {
-	fprintf(stderr, "Usage: ploop init -s SIZE [-f FORMAT] [-t FSTYPE] [-b BLOCKSIZE] DELTA\n"
+	fprintf(stderr, "Usage: ploop init -s SIZE [-f FORMAT] [-t FSTYPE] [-b BLOCKSIZE]\n"
+			"		[-B FSBLOCKSIZE] DELTA\n"
 			"       SIZE := NUMBER[kmg]\n"
 			"       FORMAT := { raw | ploop1 }\n"
 			"       DELTA := path to new image file\n"
-			"       BLOCKSIZE := block size in sectors\n"
-			"       FSTYPE := make file system\n");
+			"       FSTYPE := make file system\n"
+			"       BLOCKSIZE := cluster block size in sectors\n"
+			"       FSBLOCKSIZE := file system block size in bytes\n");
 }
 
 static int plooptool_init(int argc, char **argv)
 {
 	int i, f, ret;
 	off_t size_sec = 0;
+	char * endptr;
 	struct ploop_create_param param = {};
 
 	param.mode = PLOOP_EXPANDED_MODE;
-	while ((i = getopt(argc, argv, "s:b:f:t:")) != EOF) {
+	param.fmt_version = PLOOP_FMT_UNDEFINED;
+	while ((i = getopt(argc, argv, "s:b:B:f:t:v:")) != EOF) {
 		switch (i) {
 		case 's':
 			if (parse_size(optarg, &size_sec)) {
@@ -80,16 +84,20 @@ static int plooptool_init(int argc, char **argv)
 				return SYSEXIT_PARAM;
 			}
 			break;
-		case 'b': {
-			  char * endptr;
-
+		case 'b':
 			  param.blocksize = strtoul(optarg, &endptr, 0);
 			  if (*endptr != '\0') {
 				  usage_init();
 				  return SYSEXIT_PARAM;
 			  }
 			  break;
-		}
+		case 'B' :
+			  param.fsblocksize = strtoul(optarg, &endptr, 0);
+			  if (*endptr != '\0') {
+				  usage_init();
+				  return SYSEXIT_PARAM;
+			  }
+			  break;
 		case 'f':
 			f = parse_format_opt(optarg);
 			if (f < 0) {
@@ -107,6 +115,17 @@ static int plooptool_init(int argc, char **argv)
 				return SYSEXIT_PARAM;
 			}
 			break;
+		case 'v':
+			if (!strcmp(optarg, "1"))
+				param.fmt_version = PLOOP_FMT_V1;
+			else if (!strcmp(optarg, "2"))
+				param.fmt_version = PLOOP_FMT_V2;
+			else {
+				fprintf(stderr, "Unknown ploop image version: %s\n",
+						optarg);
+				return SYSEXIT_PARAM;
+			}
+			break;
 		default:
 			usage_init();
 			return SYSEXIT_PARAM;
@@ -281,6 +300,7 @@ static int plooptool_start(int argc, char **argv)
 
 	if (ioctl(lfd, PLOOP_IOC_START, 0) < 0) {
 		perror("PLOOP_IOC_START");
+		close(lfd);
 		return SYSEXIT_DEVIOC;
 	}
 
@@ -288,6 +308,7 @@ static int plooptool_start(int argc, char **argv)
 		perror("BLKRRPART");
 	}
 
+	close(lfd);
 	return 0;
 }
 
@@ -332,9 +353,11 @@ static int plooptool_stop(int argc, char **argv)
 
 	if (ioctl(lfd, PLOOP_IOC_STOP, 0) < 0) {
 		perror("PLOOP_IOC_STOP");
+		close(lfd);
 		return SYSEXIT_DEVIOC;
 	}
 
+	close(lfd);
 	return 0;
 }
 
@@ -379,9 +402,11 @@ static int plooptool_clear(int argc, char **argv)
 
 	if (ioctl(lfd, PLOOP_IOC_CLEAR, 0) < 0) {
 		perror("PLOOP_IOC_CLEAR");
+		close(lfd);
 		return SYSEXIT_DEVIOC;
 	}
 
+	close(lfd);
 	return 0;
 }
 
@@ -512,8 +537,11 @@ static int plooptool_rm(int argc, char **argv)
 	level = rmopts.level;
 	if (ioctl(lfd, PLOOP_IOC_DEL_DELTA, &level) < 0) {
 		perror("PLOOP_IOC_DEL_DELTA");
+		close(lfd);
 		return SYSEXIT_DEVIOC;
 	}
+
+	close(lfd);
 	return 0;
 }
 
@@ -569,14 +597,11 @@ static int plooptool_snapshot(int argc, char **argv)
 
 		ploop_free_diskdescriptor(di);
 	} else {
-		__u32 blocksize = 0;
 		if (!device) {
 			usage_snapshot();
 			return SYSEXIT_PARAM;
 		}
-		if (ploop_get_attr(device, "block_size", (int*) &blocksize))
-			return SYSEXIT_SYSFS;
-		ret = create_snapshot(device, argv[0], blocksize, syncfs);
+		ret = create_snapshot(device, argv[0], syncfs);
 	}
 
 	return ret;
@@ -913,6 +938,14 @@ static int plooptool_list(int argc, char **argv)
 		}
 	}
 
+	argc -= optind;
+	argv += optind;
+
+	if (argc != 0) {
+		usage_list();
+		return SYSEXIT_PARAM;
+	}
+
 	snprintf(fname, sizeof(fname) - 1, "/sys/block/");
 	dp = opendir(fname);
 	if (dp == NULL) {
